/**
 * @file
 * @author Petr Bravenec
 * @copyright GNU Public License
 */

#ifndef _DATABASE_TABLES_H_
#define _DATABASE_TABLES_H_

#include <QString>
#include <QDateTime>
#include <QHostAddress>
#include <QVariant>

/**
 * @brief Class describing database table LINES
 */
class DBT_LINES {
  public:
    enum Type {
        Fronius = 0,
        ModbusTCP,
        ModbusRTU,
        Unknown
        };
    int             line;
    QString         description;
    QString         device;
    QString         sn_format;
    int             speed;
    int             retries;
    int             timeout;
    Type            type;
    QString         hostname;
    int             portnumber;
    DBT_LINES() {
        line=0; speed=9600; retries=5; timeout=3000; type = Fronius; portnumber = 502;
        }
};
Q_DECLARE_METATYPE(DBT_LINES)


/**
 * @brief Class describing lines status
 *
 * Status consists of two information for each line:
 * - number of working invertors
 * - number of invertors connected to the line
 */
class DBT_LINES_STATUS {
  public:
    int         line;
    int         working;
    int         invertors;
  DBT_LINES_STATUS() {
        line=0; working=0; invertors=0;
    }
};


/**
 * @brief Class describing database table INVERTORS
 */
class DBT_INVERTORS {
  public:
    int         invertor;
    QString     description;
    int         line;
    int         address;
    bool        now_power;
    bool        now_ac_current;
    bool        now_ac_voltage;
    bool        now_ac_frequency;
    bool        now_dc_current;
    bool        now_dc_voltage;
    bool        day_energy;
    bool        day_power_maximum;
    bool        day_ac_voltage_maximum;
    bool        day_ac_voltage_minimum;
    bool        day_dc_voltage_maximum;
    bool        day_operating_hours;
    bool        total_energy;
    bool        total_power_maximum;
    bool        total_ac_voltage_maximum;
    bool        total_ac_voltage_minimum;
    bool        total_dc_voltage_maximum;
    bool        total_operating_hours;
    bool        temperature_1;
    bool        temperature_2;
    bool        irradiance;
};
Q_DECLARE_METATYPE(DBT_INVERTORS)


/**
 * @bries Class describing lines to interfacebox rellays
 */
class DBT_INTERFACEBOXES {
  public:
    int         interfacebox;
    int         line;
    QString     description;
};
Q_DECLARE_METATYPE(DBT_INTERFACEBOXES)


/**
 * @brief Class describing database table GEOCOORDINATES
 */
class DBT_GEOCOORDINATES {
  public:
    QVariant    longitude;
    QVariant    latitude;
    DBT_GEOCOORDINATES() { 
        };
};
Q_DECLARE_METATYPE(DBT_GEOCOORDINATES)


/**
 * @brief Class describing database table DATA
 */
class DBT_DATA {
  public:
    int         invertor;
    QDateTime   date;
    QString     error;
    QVariant    now_power;
    QVariant    now_ac_current;
    QVariant    now_ac_voltage;
    QVariant    now_ac_frequency;
    QVariant    now_dc_current;
    QVariant    now_dc_voltage;
    QVariant    day_energy;
    QVariant    day_power_maximum;
    QVariant    day_ac_voltage_maximum;
    QVariant    day_ac_voltage_minimum;
    QVariant    day_dc_voltage_maximum;
    QVariant    day_operating_hours;
    QVariant    total_energy;
    QVariant    total_power_maximum;
    QVariant    total_ac_voltage_maximum;
    QVariant    total_ac_voltage_minimum;
    QVariant    total_dc_voltage_maximum;
    QVariant    total_operating_hours;
    QVariant    temperature_1;
    QVariant    temperature_2;
    QVariant    irradiance;
    QString     status;
    DBT_DATA() { 
        invertor = 0;
        date = QDateTime::currentDateTime();
        };
};
Q_DECLARE_METATYPE(DBT_DATA)


/**
 * @brief Class describing database table DATA + INVERTORS
 */
class DBT_DATA_VW {
  public:
    int         line;
    int         address;
    QDateTime   date;
    QString     error;
    QVariant    now_power;
    QVariant    now_ac_current;
    QVariant    now_ac_voltage;
    QVariant    now_ac_frequency;
    QVariant    now_dc_current;
    QVariant    now_dc_voltage;
    QVariant    day_energy;
    QVariant    day_power_maximum;
    QVariant    day_ac_voltage_maximum;
    QVariant    day_ac_voltage_minimum;
    QVariant    day_dc_voltage_maximum;
    QVariant    day_operating_hours;
    QVariant    total_energy;
    QVariant    total_power_maximum;
    QVariant    total_ac_voltage_maximum;
    QVariant    total_ac_voltage_minimum;
    QVariant    total_dc_voltage_maximum;
    QVariant    total_operating_hours;
    QVariant    temperature_1;
    QVariant    temperature_2;
    QVariant    irradiance;
    QString     status;
    QString     serial_number;
    DBT_DATA_VW() { 
        line = 0;
        address = 0;
        };
};
Q_DECLARE_METATYPE(DBT_DATA_VW)

/**
 * @brief Class describing database table USERS
 */
class DBT_USERS {
  public:
    QString     id;
    QString     user;
    QString     password;
    bool        can_access_data;
    bool        can_change_settings;
};
Q_DECLARE_METATYPE(DBT_USERS)

/**
 * @brief Class describing database table MAINTENANCE
 */
class DBT_MAINTENANCE {
  public:
    int         days;
};

#endif
