/**
 * @file
 * @author Petr Bravenec
 * @copyright GNU Public License
 */
#include "msqlquery.h"
#include <QSqlError>
#include <QObject>
#include <QTime>
#include <QDebug>

MSqlQuery::~MSqlQuery() {
}

MSqlQuery::MSqlQuery() : QSqlQuery() {
}

MSqlQuery::MSqlQuery(QString q) : QSqlQuery(q) {
}

MSqlQuery::MSqlQuery(QSqlDatabase db) : QSqlQuery(db) {
}

void MSqlQuery::isError() {
    QSqlError e = QSqlQuery::lastError();
    if (e.type() == QSqlError::NoError) {
        return;
        }
    QString a = QString( tr(
        "Database message: %1\n"
        "Driver message: %2\n"
        "SQL query: %3\n"
         )) . arg(e.databaseText())
            . arg(e.driverText())
            . arg(lastQuery())
            ;
    qDebug() << a;
}


bool MSqlQuery::exec (const QString & q) {
    bool rc = QSqlQuery::exec(q);
    if (!rc) isError();
    return rc;
}

bool MSqlQuery::exec () {
    bool rc = QSqlQuery::exec();
    if (!rc) isError();
    return rc;
}

