/**
 * @file
 * @author Petr Bravenec
 * @copyright GNU Public License
 */
#ifndef MSQLQUERY_H
#define MSQLQUERY_H

#include <QObject>
#include <QSqlQuery>
#include <QSqlDatabase>
#include <QCoreApplication>


/**
 * @brief Extends class QSqlQuery - adds a diagnostics and logging
 *
 * Wrapper about QSqlQuery::exec() and QSqlQuery::exec(const QString&).
 * Original QSqlQuery runs quietly. This wrapper search for an error
 * and writes the error message to qDebug() stream.
 */
class MSqlQuery : public QSqlQuery {
    Q_DECLARE_TR_FUNCTIONS(MSqlQuery)
  public:
    MSqlQuery();
    MSqlQuery(QString);
    MSqlQuery(QSqlDatabase);

    virtual ~MSqlQuery();

  public:
    bool exec (const QString& query);
    bool exec ();

  private:
    void isError();

};

#endif
