/**
 * @file
 * @author Petr Bravenec
 * @copyright GNU Public License
 */

#include "interfacebox.h"
#include "qextserialport.h"
#include "database.h"
#include <QTimer>
#include <QDebug>

INTERFACEBOX::~INTERFACEBOX() {
}


INTERFACEBOX::INTERFACEBOX(QCoreApplication *) : QThread(0) {
    moveToThread(this);
}

void INTERFACEBOX::setDatabase(DATABASE *db) {
    m_db = db;
}

void INTERFACEBOX::start(Priority priority) {
    QTimer::singleShot(10, this, SLOT(slotInit()));
    QThread::start(priority);
}


void INTERFACEBOX::slotInit() {
    QTimer *t = new QTimer(this);   
    t->setSingleShot(false);
    t->setInterval(600000);
    t->start();
    connect(t, SIGNAL(timeout()), this, SLOT(slotThreadAlive()) );
    connect(this, SIGNAL(sigRestart()), this, SLOT(slotRestart()), Qt::QueuedConnection);
}


void INTERFACEBOX::quit() {
    if (!isRunning()) return;
    QThread::quit();
}


void INTERFACEBOX::restart() {
    emit sigRestart();
}

void INTERFACEBOX::slotRestart() {
    qDebug() << "INTERFACEBOX::restart()";
    /* sem nutno doplnit restart interfaceboxu */
    QList<DBT_INTERFACEBOXES> ifcbox = m_db->interfaceboxes();
    for (int i=0; i<ifcbox.size(); i++) {
        int line = ifcbox[i].line;
        QList<DBT_LINES> lines = m_db->lines(line);
        if (lines.size() > 0) {
            restart(lines[0]);
            }
        }
}


void INTERFACEBOX::restart(const DBT_LINES& line) {
    BaudRateType speed = BAUD9600;
    switch (line.speed) {
        case    110: speed = BAUD110; break;
        case    300: speed = BAUD300; break;
        case    600: speed = BAUD600; break;
        case   1200: speed = BAUD1200; break;
        case   2400: speed = BAUD2400; break;
        case   4800: speed = BAUD4800; break;
        case   9600: speed = BAUD9600; break;
        case  19200: speed = BAUD19200; break;
        case  38400: speed = BAUD38400; break;
        case  57600: speed = BAUD57600; break;
        case 115200: speed = BAUD115200; break;
        }
    PortSettings spset;
    spset.BaudRate    = speed;
    spset.DataBits    = DATA_8;
    spset.Parity      = PAR_NONE;
    spset.StopBits    = STOP_1;
    spset.FlowControl = FLOW_OFF;
    spset.Timeout_Millisec = line.timeout;

    QextSerialPort *port = new QextSerialPort(line.device, spset, QextSerialPort::Polling);
    port->open(QIODevice::ReadWrite);
    if (port == NULL || !port->isOpen()) {
        return;
        }

    port->write("?", 1); port->flush();
    sleep(1); port->write("1111", 4); port->flush();
    sleep(1); port->write("1111", 4); port->flush();
    sleep(4);

    port->write("?", 1); port->flush();
    sleep(1); port->write("0000", 4); port->flush();
    sleep(1); port->write("0000", 4); port->flush();
    sleep(1); port->write("0000", 4); port->flush();

    port->close();
}


void INTERFACEBOX::slotThreadAlive() {
    qDebug() << "INTERFACEBOX::slotThreadAlive() Thread is alive :=)";
}

