/**
 * @file
 * @author Petr Bravenec
 * @copyright GNU Public License
 */

#ifndef _INTERFACEBOX_H_
#define _INTERFACEBOX_H_

#include <QThread>
#include <QCoreApplication>

class DATABASE;
class DBT_LINES;

/**
 * @brief Virtual class for invertor communication
 *
 * Every derived class must implement slotInit() nd slotQuit() methods.
 *
 * As parameters the object gets the line specification in DBT_LINES object and
 * list of invertors connected to this line in DBT_INTERFACEBOXS's list.
 * One independent thread is started for every line.
 *
 * When all needed parameters are set the start() function must be called.
 *
 * Data are sent to other object using signal data(DBT_DATA);
 */
class INTERFACEBOX : public QThread {
    Q_OBJECT
  public:
   ~INTERFACEBOX();
    INTERFACEBOX(QCoreApplication *);

    /**
      * @brief Starts the thread
      *
      * Starts the thread's event loop, moves the object to that thread and calls
      * from the event loop ancestor's slotInit() function.
      */
    void            start(Priority priority = InheritPriority);
    void            setDatabase(DATABASE *);

  public slots:
    /**
      * @brief Quits the running thread
      *
      */
    void            quit();

    /**
      * @brief Restarts all connected interfaceboxes
      */
    void            restart();

    void            slotThreadAlive();

  signals:
    void            sigRestart();

  private slots:
    void            slotInit();
    void            slotRestart();

  private:
    DATABASE       *m_db;
    void            restart(const DBT_LINES&);

};

#endif
