/**
 * @file
 * @author Petr Bravenec
 * @copyright GNU Public License
 */

#ifndef _FRONIUS_request_H_
#define _FRONIUS_request_H_

#include "fronius.h"

/**
 * @brief Structure of fronius request sent to invertor over serial line
 */
class FRONIUS_request {
  public:
    unsigned char   startByte0;
    unsigned char   startByte1;
    unsigned char   startByte2;
    unsigned char   length;
    unsigned char   deviceOption;
    unsigned char   deviceNumber;
    unsigned char   command;
    unsigned char   checksum;

    const unsigned char static FRONIUS_START_VALUE;
    const unsigned char static FRONIUS_INTERFACE_CARD;
    const unsigned char static FRONIUS_SENSORBOX;
    const unsigned char static FRONIUS_INVERTOR;

    /**
     * @brief Initialize request with constant values;
     */
    FRONIUS_request(unsigned char invertor, unsigned char cmd) {
        startByte0   = FRONIUS_START_VALUE;
        startByte1   = FRONIUS_START_VALUE;
        startByte2   = FRONIUS_START_VALUE;
        length       = 0;
        command      = cmd;
        deviceOption = (command==FRONIUS::FRONIUS_CMD_GET_ACTIVE_INVERTOR_NUMBER || 
                        command==FRONIUS::FRONIUS_CMD_GET_VERSION) 
                            ? FRONIUS_INTERFACE_CARD 
                     : (command==FRONIUS::FRONIUS_CMD_GET_TEMPERATURE_1 || 
                        command==FRONIUS::FRONIUS_CMD_GET_TEMPERATURE_2 || 
                        command==FRONIUS::FRONIUS_CMD_GET_IRRADIANCE) 
                            ? FRONIUS_SENSORBOX
                            : FRONIUS_INVERTOR
                ;
        deviceNumber = (command==FRONIUS::FRONIUS_CMD_GET_ACTIVE_INVERTOR_NUMBER ||
                        command==FRONIUS::FRONIUS_CMD_GET_VERSION) 
                ? 0
                : invertor
                ;
        checksum     = calculateChecksum();
        }

    unsigned char calculateChecksum() {
        unsigned char checksum = 0;
        checksum += length;
        checksum += deviceOption;
        checksum += deviceNumber;
        checksum += command;
        return checksum;
        }

};

#endif
