/**
 * @file
 * @author Petr Bravenec
 * @copyright GNU Public License
 */

#include "fronius_response.h"


FRONIUS_response::FRONIUS_response() {
    hdr0=0; 
    hdr1=0; 
    hdr2=0; 
    length=0; 
    device_option=0; 
    device_number=0; 
    command=0;
}


/**
 * @brief Check header validity
 *
 * @returns true when the header is valid
 */
bool FRONIUS_response::isHeaderValid() {
    return (hdr0==0x80 && hdr1==0x80 && hdr2==0x80);
}

/**
 * @brief Returns checksum of the response
 */
unsigned char FRONIUS_response::checksum() {
    return buffer.buffer[length];
}


/**
 * @brief Check header validity
 *
 * @returns true when the header is valid
 */
bool FRONIUS_response::isChecksumValid() {
    unsigned char chsum = \
            length + \
            device_option + \
            device_number + \
            command;
    for (int i=0; i<length; i++) {
        chsum += buffer.buffer[i];
        }
    return (chsum == checksum());

}


/**
 * @brief Decodes and returns value stored in response
 */
QVariant FRONIUS_response::value() {
    int value;
    signed char exponent = buffer.Value.exponent;

    if (exponent == 0xb) {
        throw(QString(tr("overflow")));
        }

    if (exponent == -4) {
        throw(QString(tr("underflow")));
        }

    value  = buffer.Value.lsb;
    value += buffer.Value.msb * 256;

    if (value > 32768) {
        value = value - 65536;
        }

    if (exponent == 0) {
        return QVariant(value);
        }

    if (exponent > 0) {
        return QVariant( (int) (value * (pow(10, exponent)) ) );
        }

    if (exponent < 0) {
        return QVariant( ((double)value) * pow(10, exponent) );
        }

    return QVariant();
}


