/**
 * @file
 * @author Petr Bravenec
 * @copyright GNU Public License
 */

#ifndef _FRONIUS_response_H_
#define _FRONIUS_response_H_

#include <math.h>
#include <QCoreApplication>
#include <QString>
#include <QVariant>


/**
 * @brief Response from fronius invertor
 */
class FRONIUS_response {
    Q_DECLARE_TR_FUNCTIONS(FRONIUS_response);
  public:
    unsigned char hdr0;
    unsigned char hdr1;
    unsigned char hdr2;
    unsigned char length;
    unsigned char device_option;
    unsigned char device_number;
    unsigned char command;

    union {
        unsigned char buffer[255];

        /** 
         * @brief Structure of data payload when an error occured
         */
        struct {
            unsigned char failedCommand;
            unsigned char errorInformation;
            unsigned char checksum;
            } Error;

        /**
         * @brief Structure of data payload when answering to version command
         */
        struct {
            unsigned char ifcType;
            unsigned char versionMajor;
            unsigned char versionMinor;
            unsigned char versionRelease;
            unsigned char checksum;
            } Version;

        /**
         * @brief Structure of data payload when answering to devicetype command
         */
        struct {
            unsigned char typeIdentification;
            unsigned char checksum;
            } DeviceType;

        /**
         * @brief Structure of data payload when answering to active invertors command
         */
        struct {
            unsigned char activeInvertor;
            unsigned char checksum;
            } ActiveInvertor;

        /**
         * @brief Structure of data payload when answering command requesting a value
         */
        struct {
            unsigned char msb;
            unsigned char lsb;
            unsigned char exponent;
            unsigned char checksum;
            } Value;

        /**
         * @brief Structure of data payload when answering status command 
         */
        struct {
            unsigned char status;
            unsigned char checksum;
            } Status;

        } buffer;


    const unsigned char static HEADER_LENGTH = 7;

    FRONIUS_response(); 

    bool        isChecksumValid();
    bool        isHeaderValid();
    QVariant    value();

  private:
    unsigned char checksum() ;

};


#endif
