/**
 * @file
 * @author Petr Bravenec
 * @copyright GNU Public License
 */

#include "invertor.h"
#include "fronius.h"
#include "modbustcp.h"
#include <QTimer>
#include <QDebug>

INVERTOR::~INVERTOR() {
}


INVERTOR *INVERTOR::create(DBT_LINES::Type type) {
    switch (type) {
        case DBT_LINES::Fronius:
            return new FRONIUS();
        case DBT_LINES::ModbusTCP:
            return new MODBUSTCP();
        case DBT_LINES::ModbusRTU:
        case DBT_LINES::Unknown:
            return new FRONIUS();
        };
    return NULL;
}


INVERTOR::INVERTOR() : QThread(0) {
    moveToThread(this);
}


void INVERTOR::start(Priority priority) {
    QTimer::singleShot(10, this, SLOT(slotInit()));
    QThread::start(priority);
}


void INVERTOR::setLine(const DBT_LINES& line) {
    m_line = line;
    m_status.line             = m_line.line;
    m_status.line_description = m_line.description;
}


void INVERTOR::setInvertors(const QList<DBT_INVERTORS>& invertors) {
    m_invertors = invertors;
}


void INVERTOR::setStatus(int address, int retries, const QString& command, const QString& status) {
    QMutexLocker locker(&m_status_mutex);
    m_status.address = address;
    m_status.retries = retries;
    m_status.command = command;
    m_status.status  = status;
}


INVERTOR_status INVERTOR::status() {
    QMutexLocker locker(&m_status_mutex);
    return m_status;
}


void INVERTOR::quit() {
    if (!isRunning()) return;
    slotQuit();
    QThread::quit();
}



