/**
 * @file
 * @author Petr Bravenec
 * @copyright GNU Public License
 */

#ifndef _MODBUSTCP_H_
#define _MODBUSTCP_H_
#include <QList>
#include <QDateTime>
#include <QTimer>
#include <QVariant>

#include "invertor.h"
#include "fronius_response.h"
#include "qextserialport.h"

class DBT_DATA;

/**
 * @brief Communicates with invertors using serial port
 *
 * The class opens one serial port. If you have more ports, you have to have
 * more instances of the MODBUSTCP class;
 */
class MODBUSTCP : public INVERTOR {
    Q_OBJECT
  public:
    MODBUSTCP();
   ~MODBUSTCP();

  public slots:
    void            open();


  private:

  private slots:
    void            slotInit();
    void            slotQuit();
    void            loop();

  private:

    QTimer         *m_timer;
    quint16         m_frameid;

    void            readInvertor();
    void            readInvertorInfo(const DBT_INVERTORS& invertor);
    QByteArray      readModbus(const DBT_INVERTORS& invertor, quint16 regaddress, quint16 regcnt, quint16 expectedLength);

    void            incrementFrameId() { m_frameid++; }

    float           float32(const QByteArray& data, int offset) const;
    qint16          integer16(const QByteArray& data, int offset) const;
    qint32          integer32(const QByteArray& data, int offset) const;

    int             m_current_invertor_index;   ///< Current index in INVERTOR::m_invertors list
    int             m_number_of_ok;
    int             m_number_of_err;

    QString         stateToString(int);
    QString         exceptionCodeToString(int);



};

#endif
