/**
 * @file
 *
 * @author Petr Bravenec <petr.bravenec@hobrasoft.cz>
 */

#include "socketlist.h"

SocketList *SocketList::m_socketList = NULL;

SocketList::SocketList(QObject *parent) : QObject(parent) {
    m_socketList = this;
    m_parentThread = QThread::currentThread();
}


SocketList *SocketList::socketList(QObject *parent) {
    if (m_socketList == NULL) {
        new SocketList(parent);
        }
    return m_socketList;
}


void SocketList::addLine(int line, const QString& hostname, int port) {
    if (m_line_socket.contains(line)) {
        return;
        }

    m_line_hostname[line] = hostname;
    m_line_port    [line] = port;
    m_line_socket  [line] = NULL;

}


void SocketList::open(int line) {
    qDebug() << "SocketList::open()" << line;

    if (m_parentThread != QThread::currentThread()) {
        qFatal("Parent's thread not equals to currentThread in SocketList::open");
        }

    if (!m_line_socket.contains(line)) {
        qFatal("SocketList::open(line) line does not exist");
        }

    if (m_line_socket[line] != NULL) {
        QTcpSocket *socket = m_line_socket[line];
        socket->close();
        socket->deleteLater();
    	qDebug() << "SocketList::open() delete socket" << socket;
        }

    QTcpSocket *socket = new QTcpSocket(this);
    socket->connectToHost( m_line_hostname[line], m_line_port[line]);
    if (!socket->waitForConnected()) {
	qDebug() << "SocketList::open() could not open to host" << m_line_hostname[line] << m_line_port[line] << socket;
        socket->deleteLater();
        throw tr("Could not connect to %1:%2").arg(m_line_hostname[line]).arg(m_line_port[line]);
        }

    m_line_socket[line] = socket;
}


QTcpSocket *SocketList::socket(int line) {
    if (m_parentThread != QThread::currentThread()) {
        qFatal("Parent's thread not equals to currentThread in SocketList::open");
        }

    if (!m_line_socket.contains(line)) {
        throw tr("Socket for line %1 does not exists").arg(line);
        }

    QTcpSocket *socket = m_line_socket[line];
    if (socket == NULL) {
        open(line);
        socket = m_line_socket[line];
        }

    return socket;
}


