/**
 * @file
 * 
 * @author Petr Bravenec <petr.bravenec@hobrasoft.cz>
 */
#ifndef _SocketList_H_
#define _SocketList_H_

#include <QObject>
#include <QTcpSocket>
#include <QHash>
#include <QThread>

#define SOCKETLIST SocketList::socketList()

/**
 * @brief
 */
class SocketList : public QObject {
    Q_OBJECT
  public:
    static SocketList *socketList(QObject *parent = NULL);

    /**
     * @brief Returns socket associated with line
     *
     * @throws QString when an error occured. Application should call open(int) to recover from error
     */
    QTcpSocket *socket(int line);

    /**
     * @brief Adds new line to list 
     *
     * Socket is created and opened later since it runs in different thread
     */
    void        addLine(int line, const QString& hostname, int port);

    /**
     * @brief Opens socket for line
     *
     * @throws QString when en arror occured. Application can call open(int) again to recover from error
     * 
     * If the line does not exist, the application fails
     */
    void        open(int line);

  private:
    SocketList(QObject *parent);
    static SocketList *m_socketList;

    QHash<int, QTcpSocket *> m_line_socket;
    QHash<int, QString>      m_line_hostname;
    QHash<int, int>          m_line_port;

    QThread	*m_parentThread;
};

#endif

