/**
 * @file
 * @author Petr Bravenec
 * @copyright GNU Public License
 */

/**
  @mainpage

  Application is composed from few parts:
    - Main. Starts all needed threads. Runs as a daemon in unix or as a service in Windows environment.
    - Database. SQLight is used. Becase the database does not work in multihread environment,
      there must be only one common interface accessible via slots and signals from other threads.
      The database functionality is covered by DATABASE object. Database runs within main thread.
    - Invertors. There is one common interface - virtual class INVERTOR. Every invertor's communication
      protocol is based on this class. Every object runs in standalone thread. There can be multiple
      objects of the same class - one for every communication line. At this time only Fronius 
      communication protocol is implemented.
    - Webserver. The QtWebApp library is used with small modifications for Qt 4.6. All configuration
      and data communication is made via this object.

*/

#include <QDebug>
#include "startup.h"
#include "database_tables.h"
#include <sys/types.h>
#include <unistd.h>
#include <pwd.h>

/** Name of this application */
#define APPNAME "Nekardas"

/** Publisher of this application */
#define ORGANISATION "Hobrasoft"

/** Short description of this application */
#define DESCRIPTION "Applications gets data from fronius invertors"

/**
  Entry point of the program.
  Passes control to the service helper.
*/
int main(int argc, char *argv[]) {

    char *username = NULL;
    for (int a=0; a<argc; a++) {
        if (QString(argv[a]) == "-U") {
            if (++a >= argc) break;
            username = argv[a];
            }
        }

    if (username != NULL) {
        struct passwd *spw;
        spw = getpwnam(username);
        if (spw != NULL) {
            int unused;
            Q_UNUSED(unused);
            unused = seteuid(spw->pw_uid);
            unused = setegid(spw->pw_gid);
            unused = setgid(spw->pw_gid);
          } else {
            qFatal( QString("Cannot change effective user to %1").arg(username).toUtf8() );
            exit(2);
            }
        }

    qRegisterMetaType<DBT_DATA>("DBT_DATA");

    // Use a qtservice wrapper to start the application as a Windows service or Unix daemon
    Startup startup(argc, argv);
    return startup.exec();

}

