#ifndef _SLUNCE_H_
#define _SLUNCE_H_

#include <QDateTime>
#include <QList>
#include <QThread>
#include <QMutex>
#include <QCoreApplication>


/**
 * @brief Astronomické výpočty polohy slunce
 *
 * Ze zadaného času dokáže spočítat polohu slunce na obloze.
 * Pro panel v zadané rovině (sklon a natočení) spočítá úhel dopadu.
 * Provádí základní simulace atmosféry.
 */
class SLUNCE : public QThread {
    Q_OBJECT
  public:
   ~SLUNCE();
    SLUNCE(QCoreApplication *);

    const static double DTR = 57.295779513;

    void        setObserver(double lat, double lng);

    double      ra ();    ///< Vrací rektascenzi slunce
    double      dec();    ///< Vrací deklinaci slunce
    double      alt();    ///< Vrací výšku slunce nad obzorem
    double      az ();    ///< Vrací azimut slunce
    QString     toDeg  (double);

    void        start(Priority priority = InheritPriority);

  private slots:
    void        slotInit();
    void        spocti();

  private:
    void        sun_ra_dec();
    void        sun_alt_az();
    void        sun_uhel();
    double      to360 (double);
    double      to24  (double);

    QDateTime   datum;
    double      datum_jd;
    double      sun_ra;
    double      sun_dec;
    double      sun_alt;
    double      sun_az;
    double      obs_long;
    double      obs_lat;

    QMutex      m_mutex;


};

#endif
