/**
 * @file
 * @copyright GNU Public License
 * @author Jiří Procházka
*/

#include "exportcontroller.h"
#include <QVariant>
#include <QDateTime>
#include <QHostInfo>

ExportController::ExportController() {
    m_needs_access_data     = true;
    m_needs_change_settings = false;
}

void ExportController::service(HttpRequest& request, HttpResponse& response) {

    if (needsLogin(request, response)) {
        return;
        }

    QString body ("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
        "<?xml-stylesheet type=\"text/xsl\" href=\"data.xsl\"?>\n"
        "<actual_state_of_invertors>\n");
    body.append(QString("<FVEname>%1</FVEname>\n")
        .arg(QHostInfo::localHostName() + "." + QHostInfo::localDomainName())
        );

    QDateTime from, to;
    QString fromString (QByteArray::fromPercentEncoding(request.getParameter("from")));
    QString toString (QByteArray::fromPercentEncoding(request.getParameter("to")));
    if(!fromString.isEmpty()) from = QDateTime::fromString(fromString, "yyyy-MM-dd hh:mm:ss");
    if(!toString.isEmpty()) to = QDateTime::fromString(toString, "yyyy-MM-dd hh:mm:ss");

    QList<DBT_DATA_VW> data = Static::db->data(from, to);

    QList<DBT_DATA_VW>::const_iterator i;
    for(i = data.constBegin(); i != data.constEnd(); ++i) {
        const DBT_DATA_VW& inv = *i;

        if (inv.address >= 1000) {

            // SENSORS
            body.append("<sensors>\n");
            body.append(QString("\t<line>%1</line>\n").arg(inv.line));
            body.append(QString("\t<address>%1</address>\n").arg(inv.address));
            if(!inv.serial_number.isNull())
                body.append(QString("\t<sn>%1</sn>\n").arg(inv.serial_number));
            if(!inv.date.isNull())
                body.append(QString("\t<date>%1</date>\n").arg(inv.date.toString("yyyy-MM-dd hh:mm:ss")));
            if(!inv.error.isNull())
                body.append(QString("\t<error>%1</error>\n").arg(inv.error));
            if(!inv.temperature_1.isNull())
                body.append(QString("\t<temperature_1>%1</temperature_1>\n").arg(inv.temperature_1.toString()));
            if(!inv.temperature_2.isNull())
                body.append(QString("\t<temperature_2>%1</temperature_2>\n").arg(inv.temperature_2.toString()));
            if(!inv.irradiance.isNull())
                body.append(QString("\t<irradiance>%1</irradiance>\n").arg(inv.irradiance.toString()));
            body.append("</sensors>\n");


          } else {

            // INVERTORS
            body.append("<invertor>\n");
            body.append(QString("\t<line>%1</line>\n").arg(inv.line));
            body.append(QString("\t<address>%1</address>\n").arg(inv.address));
            if(!inv.serial_number.isNull())
                body.append(QString("\t<sn>%1</sn>\n").arg(inv.serial_number));
            if(!inv.date.isNull())
                body.append(QString("\t<date>%1</date>\n").arg(inv.date.toString("yyyy-MM-dd hh:mm:ss")));
            if(!inv.error.isNull())
                body.append(QString("\t<error>%1</error>\n").arg(inv.error));
            if(!inv.now_power.isNull())
                body.append(QString("\t<now-power>%1</now-power>\n").arg(inv.now_power.toString()));
            if(!inv.now_ac_current.isNull())
                body.append(QString("\t<now-ac-current>%1</now-ac-current>\n").arg(inv.now_ac_current.toString()));
            if(!inv.now_ac_voltage.isNull())
                body.append(QString("\t<now-ac-voltage>%1</now-ac-voltage>\n").arg(inv.now_ac_voltage.toString()));
            if(!inv.now_ac_frequency.isNull())
                body.append(QString("\t<now-ac-frequency>%1</now-ac-frequency>\n").arg(inv.now_ac_frequency.toString()));
            if(!inv.now_dc_current.isNull())
                body.append(QString("\t<now-dc-current>%1</now-dc-current>\n").arg(inv.now_dc_current.toString()));
            if(!inv.now_dc_voltage.isNull())
                body.append(QString("\t<now-dc-voltage>%1</now-dc-voltage>\n").arg(inv.now_dc_voltage.toString()));
            if(!inv.day_energy.isNull())
                body.append(QString("\t<day-energy>%1</day-energy>\n").arg(inv.day_energy.toString()));
            if(!inv.day_power_maximum.isNull())
                body.append(QString("\t<day-power-maximum>%1</day-power-maximum>\n").arg(inv.day_power_maximum.toString()));
            if(!inv.day_ac_voltage_maximum.isNull())
                body.append(QString("\t<day-ac-voltage-maximum>%1</day-ac-voltage-maximum>\n").arg(inv.day_ac_voltage_maximum.toString()));
            if(!inv.day_ac_voltage_minimum.isNull())
                body.append(QString("\t<day-ac-voltage-minimum>%1</day-ac-voltage-minimum>\n").arg(inv.day_ac_voltage_minimum.toString()));
            if(!inv.day_dc_voltage_maximum.isNull())
                body.append(QString("\t<day-dc-voltage-maximum>%1</day-dc-voltage-maximum>\n").arg(inv.day_dc_voltage_maximum.toString()));
            if(!inv.day_operating_hours.isNull())
                body.append(QString("\t<day-operating-hours>%1</day-operating-hours>\n").arg(inv.day_operating_hours.toString()));
            if(!inv.total_energy.isNull())
                body.append(QString("\t<total-energy>%1</total-energy>\n").arg(inv.total_energy.toString()));
            if(!inv.total_power_maximum.isNull())
                body.append(QString("\t<total-power-maximum>%1</total-power-maximum>\n").arg(inv.total_power_maximum.toString()));
            if(!inv.total_ac_voltage_maximum.isNull())
                body.append(QString("\t<total-ac-voltage-maximum>%1</total-ac-voltage-maximum>\n").arg(inv.total_ac_voltage_maximum.toString()));
            if(!inv.total_ac_voltage_minimum.isNull())
                body.append(QString("\t<total-ac-voltage-minimum>%1</total-ac-voltage-minimum>\n").arg(inv.total_ac_voltage_minimum.toString()));
            if(!inv.total_dc_voltage_maximum.isNull())
                body.append(QString("\t<total-dc-voltage-maximum>%1</total-dc-voltage-maximum>\n").arg(inv.total_dc_voltage_maximum.toString()));
            if(!inv.total_operating_hours.isNull())
                body.append(QString("\t<total-operating-hours>%1</total-operating-hours>\n").arg(inv.total_operating_hours.toString()));
            body.append(QString("\t<status>%1</status>\n").arg(inv.status));
            body.append("</invertor>\n");
            }

    }
    body.append("</actual_state_of_invertors>\n");

    QByteArray databody;
    QByteArray contenttype;
    QByteArray zip = request.getParameter("zip");
    if (zip == "Y" || zip == "y") {
        contenttype = "application/x-gzip";
        databody = qCompress(body.toUtf8(),9);
      } else {
        contenttype = "text/xml; charset=utf-8";
        databody = body.toUtf8();
        }

    response.setHeader("Content-Type", contenttype);
    response.setHeader("Cache-Control", "private, must-revalidate, max-age=0");
    response.setHeader("Pragma", "no-cache");
    response.write(databody);
}


