/**
 * @file
 * @copyright GNU Public License
 * @author Petr Bravenec, Jiří Procházka
*/

#include "interfaceboxescontroller.h"
#include "static.h"
#include <QVariant>
#include <QDateTime>

InterfaceboxesController::InterfaceboxesController() {
    qRegisterMetaType<DBT_INTERFACEBOXES>();
    connect(this, SIGNAL(saveInterfacebox(DBT_INTERFACEBOXES)), Static::db, SLOT(saveInterfacebox(DBT_INTERFACEBOXES)), Qt::QueuedConnection);
    connect(this, SIGNAL(deleteInterfacebox(int)), Static::db, SLOT(deleteInterfacebox(int)), Qt::QueuedConnection);
    connect(this, SIGNAL(restartInterfacebox()), Static::interfacebox, SLOT(restart()), Qt::QueuedConnection);
}

void InterfaceboxesController::servicePrivate(HttpRequest& request) {
    QByteArray action = request.getParameter("action");
    if(action.startsWith("add")) {
        actionAdd(request); return;
    } else if(action.startsWith("edit")) {
        actionEdit(request); return;
    } else if(action.startsWith("delete")) {
        actionDelete(request); return;
    } else if(action.startsWith("restart")) {
        actionRestart(request); return;
    }

    /** Default action/page */

    QList<DBT_INTERFACEBOXES> interfaceboxes = Static::db->interfaceboxes();
    write(QString("<h2>%1</h2>\n").arg(tr("Interfaceboxes")));
    write(QString("<a href=\"/interfaceboxes?action=add\">%1</a>\n").arg(tr("Add an interfacebox")));
    write("<table>\n");
    write( QString("<tr class=\"line1st\"><th></th><th>%1</th><th>%2</th>""</tr>")
            .arg(tr("Description"))
            .arg(tr("Line"))
         ); 

    QString s2 ("<tr>"
            "<td><a href=\"/interfaceboxes?action=edit&id=%4\">%1</a> " 
                "<a href=\"/interfaceboxes?action=delete&id=%4\">%2</a></td>" 
            "<td>%3</td><td>%5</td></tr>");
    QList<DBT_INTERFACEBOXES>::const_iterator i;
    for(i = interfaceboxes.constBegin(); i != interfaceboxes.constEnd(); ++i) {
        const DBT_INTERFACEBOXES& l = *i;
        write(QString(s2)
                .arg(tr("Edit"), tr("Delete"))
                .arg(l.description)
                .arg(l.interfacebox)
                .arg(l.line)
             );
    }

    write("</table>\n");
}

QString InterfaceboxesController::getEditFormContents(HttpRequest& request, const DBT_INTERFACEBOXES& interfacebox, bool generator) {
    Q_UNUSED(request);
    Q_UNUSED(generator);
    QString s;
    s.append(QString("<input type=\"hidden\" name=\"id\" value=\"%1\" />\n").arg(interfacebox.interfacebox));
    QString i ("<tr><td><label for=\"%2\">%1</label>: </td><td><input type=\"text\" name=\"%2\" value=\"%3\" /></td></tr>\n");
    s.append(i.arg(tr("Description"), "description", interfacebox.description));

    // line selectbox
    QList<DBT_LINES> lines = Static::db->lines();
    s.append(QString("<tr><td><label for=\"%2\">%1</label>: </td><td><select id=\"%2\" name=\"%2\">\n").arg(tr("Line"), "line"));
    QList<DBT_LINES>::const_iterator ii;
    for(ii = lines.constBegin(); ii != lines.constEnd(); ++ii) {
        const DBT_LINES& l = *ii;
        s.append(QString("<option value=\"%1\" %4>%1 %2 %3</option>\n")
                .arg(l.line).arg(l.device, l.description, 
                    (l.line == interfacebox.line) ? "selected=\"selected\"" : ""));
    }
    s.append("</select></td></tr>\n");

    return s;
}

void InterfaceboxesController::readForm(DBT_INTERFACEBOXES& interfacebox, HttpRequest& request) {
    interfacebox.interfacebox   = request.getParameter("id").toInt();
    interfacebox.description    = request.getParameter("description");
    interfacebox.line           = request.getParameter("line").toInt();
}

/**
 * Add an interfacebox action
 */
void InterfaceboxesController::actionAdd(HttpRequest& request) {
    qDebug() << "InterfaceboxesController::actionAdd()";
    QByteArray action = request.getParameter("action");
    /// a form
    write(QString("<h2>%1</h2>\n").arg(tr("Add an interfacebox")));
    write("<form method=\"post\">\n"
        "<input type=\"hidden\" name=\"action\" value=\"add2\" />\n");
    DBT_INTERFACEBOXES interfacebox;
    readForm(interfacebox, request);
    write("<table class=\"formTable\">");
    write(getEditFormContents(request));
    write("</table>");
    write(QString("<input type=\"submit\" value=\"%1\">\n"
    "</form>\n").arg(tr("Add")));
    if(action == "add2") {
        /// second page doing the action and redirecting
        interfacebox.interfacebox= 0;
        /// validation
        if(true) {
            emit saveInterfacebox(interfacebox);
            QString location = QString("/interfaceboxes?message=%1").arg(percentEncode(tr("Interfacebox added.")));
            redirect(location);
        } else {
            write(QString("<p>%1</p>").arg(tr("Interfacebox definition invalid (not added).")));
        }
    }
}

/**
 * Edit interfacebox action
 */
void InterfaceboxesController::actionEdit(HttpRequest& request) {
    QByteArray action = request.getParameter("action");
    DBT_INTERFACEBOXES interfacebox;
    /// a form
    write(QString("<h2>%1</h2>\n").arg(tr("Edit an interfacebox")));
    write("<form method=\"post\">\n"
        "<input type=\"hidden\" name=\"action\" value=\"edit2\" />\n");
    if(action == "edit") {
        QByteArray id = request.getParameter("id");
        int idi = id.toInt();
        if(id.isEmpty() || idi < 1) {
            redirect(QString("/interfaceboxes?message=%1").arg(percentEncode(tr("Error: Invalid interfacebox ID."))));
            return;
        }
        QList<DBT_INTERFACEBOXES> interfaceboxes = Static::db->interfaceboxes(QList<int>() << idi);
        if(interfaceboxes.empty()) {
            redirect(QString("/interfaceboxes?message=%1").arg(percentEncode(tr("Error: no such interfacebox found."))));
            return;
        }
        write("<table class=\"formTable\">");
        write(getEditFormContents(request, interfaceboxes.at(0)));
        write("</table>");
    } else {
        readForm(interfacebox, request);
        write("<table class=\"formTable\">");
        write(getEditFormContents(request, interfacebox));
        write("</table>");
    }
    write(QString("<input type=\"submit\" value=\"%1\">\n"
    "</form>\n").arg(tr("Edit")));
    if(action == "edit2") {
        /// second page doing the action and redirecting
        /// validation
        if(interfacebox.interfacebox> 0 && true) {
            emit saveInterfacebox(interfacebox);
            QString location = QString("/interfaceboxes?message=%1").arg(percentEncode(tr("Interfacebox edited.")));
            redirect(location);
        } else {
            write(QString("<p>%1</p>").arg(tr("Error: Interfacebox definition invalid (not edited).")));
        }
    }
}

/**
 * Delete itnerfacebox action
 */
void InterfaceboxesController::actionDelete(HttpRequest& request) {
    write(QString("<h2>%1</h2>\n").arg(tr("Delete an interfacebox")));
        QString location;
        QByteArray id = request.getParameter("id");
        int idi = id.toInt();
        if(!id.isEmpty() && idi > 0) {
            emit deleteInterfacebox(idi);
            location = QString("/interfaceboxes?message=%1").arg(percentEncode(tr("Interfacebox deleted.")));
        } else {
            location = QString("/interfaceboxes?message=%1").arg(percentEncode(tr("Error: Invalid invertor ID. Interfacebox not deleted.")));
        }
        redirect(location);
}


/**
 * Restarts all administered Interfaceboxes
 */
void InterfaceboxesController::actionRestart(HttpRequest& request) {
    Q_UNUSED(request);
    write(QString("<h2>%1</h2>\n")
        .arg(tr("All interfaceboxes were signalled to restart"))
        );
    emit restartInterfacebox();
}

