/**
 * @file
 * @copyright GNU Public License
 * @author Petr Bravenec, Jiří Procházka
*/

#include "invertorscontroller.h"
#include "static.h"
#include <QVariant>
#include <QDateTime>

InvertorsController::InvertorsController() {
    qRegisterMetaType<DBT_INVERTORS>();
    connect(this, SIGNAL(saveInvertor(DBT_INVERTORS)), Static::db, SLOT(saveInvertor(DBT_INVERTORS)), Qt::QueuedConnection);
    connect(this, SIGNAL(deleteInvertor(int)), Static::db, SLOT(deleteInvertor(int)), Qt::QueuedConnection);
}

void InvertorsController::servicePrivate(HttpRequest& request) {
    QByteArray action = request.getParameter("action");
    if(action.startsWith("add")) {
        actionAdd(request); return;
    } else if(action.startsWith("edit")) {
        actionEdit(request); return;
    } else if(action.startsWith("delete")) {
        actionDelete(request); return;
    }

    /** Default action/page */

    QList<DBT_INVERTORS> invertors = Static::db->invertors();
    write(QString("<h2>%1</h2>\n").arg(tr("Invertors")));
    write(QString("<a href=\"/invertors?action=add\">%1</a>\n").arg(tr("Add an invertor")));
    write("<table>\n");
    QString s0 ("<tr class=\"line1st\"><th colspan=\"4\"></th><th colspan=\"6\">%1</th><th colspan=\"6\">%2</th><th colspan=\"6\">%3</th><th colspan=\"3\">%4</th></tr>");
    s0 = s0.arg(tr("Now"), tr("Day"), tr("Total"), tr("Sensors"));
    QString s ("<tr class=\"line2nd\"><th></th>"
            "<th>%1</th>"
            "<th>%2</th>"
            "<th>%3</th>"
    "<th class=\"group\">%4</th>"
    "<th>AC [V]</th>"
    "<th>AC [A]</th>"
    "<th>AC [Hz]</th>"
    "<th>DC [V]</th>"
    "<th>DC [A]</th>"
    "<th class=\"group\">%5</th>"
    "<th>%6</th>"
    "<th>AC max [V]</th>"
    "<th>AC min [V]</th>"
    "<th>DC max [V]</th>"
    "<th>%7</th>"
    "<th class=\"group\">%5</th>"
    "<th>%6</th>"
    "<th>AC max [V]</th>"
    "<th>AC min [V]</th>"
    "<th>DC max [V]</th>"
    "<th>%7</th>"
    "<th class=\"group\">T1 [&#176;C]</th>"
    "<th>T2 [&#176;C]</th>"
    "<th>Irr [W/m&#179;]</th>"
    "</tr>");
    s = s.arg(tr("Description"), tr("Line"), tr("Addr"), tr("Pwr [W]"), tr("Energy"), tr("Pwr max [W]"), tr("Hours"));
    write(s0);
    write(s);

    QString s2 ("<tr>"
            "<td>"
                "<a href=\"/invertors?action=edit&id=%1\">%26</a> "
                "<a href=\"/invertors?action=delete&id=%1\">%27</a>"
            "</td>");
    for(int k = 2; k <= 25; k++)
        s2.append(QString("<td>\%%1</td>").arg(k));
    s2.append("</tr>\n");
    QList<DBT_INVERTORS>::const_iterator i;
    for(i = invertors.constBegin(); i != invertors.constEnd(); ++i) {
        const DBT_INVERTORS& l = *i;
        write(QString(s2)
                .arg(l.invertor)
                .arg(l.description)
                .arg(l.line)
                .arg(l.address)
                .arg((l.now_power) ? "&#x2713;" : "")
                .arg((l.now_ac_current) ? "&#x2713;" : "")
                .arg((l.now_ac_voltage) ? "&#x2713;" : "")
                .arg((l.now_ac_frequency) ? "&#x2713;" : "")
                .arg((l.now_dc_current) ? "&#x2713;" : "")
                .arg((l.now_dc_voltage) ? "&#x2713;" : "")
                .arg((l.day_energy) ? "&#x2713;" : "")
                .arg((l.day_power_maximum) ? "&#x2713;" : "")
                .arg((l.day_ac_voltage_maximum) ? "&#x2713;" : "")
                .arg((l.day_ac_voltage_minimum) ? "&#x2713;" : "")
                .arg((l.day_dc_voltage_maximum) ? "&#x2713;" : "")
                .arg((l.day_operating_hours) ? "&#x2713;" : "")
                .arg((l.total_energy) ? "&#x2713;" : "")
                .arg((l.total_power_maximum) ? "&#x2713;" : "")
                .arg((l.total_ac_voltage_maximum) ? "&#x2713;" : "")
                .arg((l.total_ac_voltage_minimum) ? "&#x2713;" : "")
                .arg((l.total_dc_voltage_maximum) ? "&#x2713;" : "")
                .arg((l.total_operating_hours) ? "&#x2713;" : "")
                .arg((l.temperature_1) ? "&#x2713;" : "")
                .arg((l.temperature_2) ? "&#x2713;" : "")
                .arg((l.irradiance) ? "&#x2713;" : "")
                .arg(tr("Edit"), tr("Delete")));
    }

    write("</table>\n");
}

QString InvertorsController::getEditFormContents(HttpRequest& request, DBT_INVERTORS invertor, bool generator) {
    QString s;
    if(generator) {
        // number of invertors textbox
        QString iii ("<tr><td><label for=\"%2\">%1</label>: </td><td><input type=\"%3\" name=\"%2\" %4 /></td></tr>\n");
        QByteArray gener = request.getParameter("generate");
        int gen = (gener.isEmpty()) ? 0 : gener.toInt();
        s.append(iii.arg(tr("Number of invertors"), "generate", "text", QString("value=\"%1\"").arg(gen)));
    }
    s.append(QString("<input type=\"hidden\" name=\"id\" value=\"%1\" />\n").arg(invertor.invertor));
    QString i ("<tr><td><label for=\"%2\">%1</label>: </td><td><input type=\"text\" name=\"%2\" value=\"%3\" /></td></tr>\n");
    if(!generator) {
        s.append(i.arg(tr("Description"), "idescription").arg(invertor.description));
        // line selectbox
        QList<DBT_LINES> lines = Static::db->lines();
        s.append(QString("<tr><td><label for=\"%2\">%1</label>: </td><td><select id=\"%2\" name=\"%2\">\n").arg(tr("Line"), "line"));
        QList<DBT_LINES>::const_iterator ii;
        for(ii = lines.constBegin(); ii != lines.constEnd(); ++ii) {
            const DBT_LINES& l = *ii;
            s.append(QString("<option value=\"%1\" %4>%1 %2 %3</option>\n")
                    .arg(l.line).arg(l.device, l.description, 
                        (l.line == invertor.line) ? "selected=\"selected\"" : ""));
        }
        s.append("</select></td></tr>\n");

        s.append(i.arg(tr("Address (sensors +1000)"), "address").arg(invertor.address));
    } else {
        s.append(i.arg(tr("First invertor address"), "address").arg(invertor.address));
    }
    i = QString("<tr><td><label for=\"%2\">%1</label>: </td><td><input type=\"checkbox\" name=\"%2\" id=\"%2\" %3 /></td></tr>\n");
    s.append(i.arg(tr("Now Power"), "now_power").arg((invertor.now_power) ? "checked=\"checked\"" : ""));
    s.append(i.arg(tr("Now AC Current"), "now_ac_current").arg((invertor.now_ac_current) ? "checked=\"checked\"" : ""));
    s.append(i.arg(tr("Now AC Voltage"), "now_ac_voltage").arg((invertor.now_ac_voltage) ? "checked=\"checked\"" : ""));
    s.append(i.arg(tr("Now AC Frequency"), "now_ac_frequency").arg((invertor.now_ac_frequency) ? "checked=\"checked\"" : ""));
    s.append(i.arg(tr("Now DC Current"), "now_dc_current").arg((invertor.now_dc_current) ? "checked=\"checked\"" : ""));
    s.append(i.arg(tr("Now DC Voltage"), "now_dc_voltage").arg((invertor.now_dc_voltage) ? "checked=\"checked\"" : ""));
    s.append(i.arg(tr("Day Energy"), "day_energy").arg((invertor.day_energy) ? "checked=\"checked\"" : ""));
    s.append(i.arg(tr("Day Power Maximum"), "day_power_maximum").arg((invertor.day_power_maximum) ? "checked=\"checked\"" : ""));
    s.append(i.arg(tr("Day AC Voltage Maximum"), "day_ac_voltage_maximum").arg((invertor.day_ac_voltage_maximum) ? "checked=\"checked\"" : ""));
    s.append(i.arg(tr("Day AC Voltage Minimum"), "day_ac_voltage_minimum").arg((invertor.day_ac_voltage_minimum) ? "checked=\"checked\"" : ""));
    s.append(i.arg(tr("Day DC Voltage Maximum"), "day_dc_voltage_maximum").arg((invertor.day_dc_voltage_maximum) ? "checked=\"checked\"" : ""));
    s.append(i.arg(tr("Day Operating Hours"), "day_operating_hours").arg((invertor.day_operating_hours) ? "checked=\"checked\"" : ""));
    s.append(i.arg(tr("Total Energy"), "total_energy").arg((invertor.total_energy) ? "checked=\"checked\"" : ""));
    s.append(i.arg(tr("Total Power Maximum"), "total_power_maximum").arg((invertor.total_power_maximum) ? "checked=\"checked\"" : ""));
    s.append(i.arg(tr("Total AC Voltage Maximum"), "total_ac_voltage_maximum").arg((invertor.total_ac_voltage_maximum) ? "checked=\"checked\"" : ""));
    s.append(i.arg(tr("Total AC Voltage Minimum"), "total_ac_voltage_minimum").arg((invertor.total_ac_voltage_minimum) ? "checked=\"checked\"" : ""));
    s.append(i.arg(tr("Total DC Voltage Maximum"), "total_dc_voltage_maximum").arg((invertor.total_dc_voltage_maximum) ? "checked=\"checked\"" : ""));
    s.append(i.arg(tr("Total Operating Hours"), "total_operating_hours").arg((invertor.total_operating_hours) ? "checked=\"checked\"" : ""));
    s.append(i.arg(tr("Temperature ch. 1"), "temperature_1").arg((invertor.temperature_1) ? "checked=\"checked\"" : ""));
    s.append(i.arg(tr("Temperature ch. 2"), "temperature_2").arg((invertor.temperature_2) ? "checked=\"checked\"" : ""));
    s.append(i.arg(tr("Irradiance"),        "irradiance").arg((invertor.irradiance) ? "checked=\"checked\"" : ""));

    return s;
}

void InvertorsController::readForm(DBT_INVERTORS& invertor, HttpRequest& request) {
    invertor.invertor = request.getParameter("id").toInt();
    invertor.description = request.getParameter("idescription");
    invertor.line = request.getParameter("line").toInt();
    invertor.address = request.getParameter("address").toInt();
    invertor.now_power = !request.getParameter("now_power").isEmpty();
    invertor.now_ac_current = !request.getParameter("now_ac_current").isEmpty();
    invertor.now_ac_voltage = !request.getParameter("now_ac_voltage").isEmpty();
    invertor.now_ac_frequency = !request.getParameter("now_ac_frequency").isEmpty();
    invertor.now_dc_current = !request.getParameter("now_dc_current").isEmpty();
    invertor.now_dc_voltage = !request.getParameter("now_dc_voltage").isEmpty();
    invertor.day_energy = !request.getParameter("day_energy").isEmpty();
    invertor.day_power_maximum = !request.getParameter("day_power_maximum").isEmpty();
    invertor.day_ac_voltage_maximum = !request.getParameter("day_ac_voltage_maximum").isEmpty();
    invertor.day_ac_voltage_minimum = !request.getParameter("day_ac_voltage_minimum").isEmpty();
    invertor.day_dc_voltage_maximum = !request.getParameter("day_dc_voltage_maximum").isEmpty();
    invertor.day_operating_hours = !request.getParameter("day_operating_hours").isEmpty();
    invertor.total_energy = !request.getParameter("total_energy").isEmpty();
    invertor.total_power_maximum = !request.getParameter("total_power_maximum").isEmpty();
    invertor.total_ac_voltage_maximum = !request.getParameter("total_ac_voltage_maximum").isEmpty();
    invertor.total_ac_voltage_minimum = !request.getParameter("total_ac_voltage_minimum").isEmpty();
    invertor.total_dc_voltage_maximum = !request.getParameter("total_dc_voltage_maximum").isEmpty();
    invertor.total_operating_hours  = !request.getParameter("total_operating_hours").isEmpty();
    invertor.temperature_1          = !request.getParameter("temperature_1").isEmpty();
    invertor.temperature_2          = !request.getParameter("temperature_2").isEmpty();
    invertor.irradiance             = !request.getParameter("irradiance").isEmpty();
}

/**
 * Add an invertor action
 */
void InvertorsController::actionAdd(HttpRequest& request) {
    QByteArray action = request.getParameter("action");
    /// a form
    write(QString("<h2>%1</h2>\n").arg(tr("Add an invertor")));
    write("<form method=\"post\">\n"
        "<input type=\"hidden\" name=\"action\" value=\"add2\" />\n");
    DBT_INVERTORS invertor;
    readForm(invertor, request);
    write("<table class=\"formTable\">");
    write(getEditFormContents(request));
    write("</table>");
    write(QString("<input type=\"submit\" value=\"%1\">\n"
    "</form>\n").arg(tr("Add")));
    if(action == "add2") {
        /// second page doing the action and redirecting
        invertor.invertor = 0;
        /// validation
        if(true) {
            emit saveInvertor(invertor);
            QString location = QString("/invertors?message=%1").arg(percentEncode(tr("Invertor added.")));
            redirect(location);
        } else {
            write(QString("<p>%1</p>").arg(tr("Invertor definition invalid (not added).")));
        }
    }
}

/**
 * Edit invertor action
 */
void InvertorsController::actionEdit(HttpRequest& request) {
    QByteArray action = request.getParameter("action");
    DBT_INVERTORS invertor;
    /// a form
    write(QString("<h2>%1</h2>\n").arg(tr("Edit an invertor")));
    write("<form method=\"post\">\n"
        "<input type=\"hidden\" name=\"action\" value=\"edit2\" />\n");
    if(action == "edit") {
        QByteArray id = request.getParameter("id");
        int idi = id.toInt();
        if(id.isEmpty() || idi < 1) {
            redirect(QString("/invertors?message=%1").arg(percentEncode(tr("Error: Invalid invertor ID."))));
            return;
        }
        QList<DBT_INVERTORS> invertors = Static::db->invertors(QList<int>() << idi);
        if(invertors.empty()) {
            redirect(QString("/invertors?message=%1").arg(percentEncode(tr("Error: no such invertor found."))));
            return;
        }
        write("<table class=\"formTable\">");
        write(getEditFormContents(request, invertors.at(0)));
        write("</table>");
    } else {
        readForm(invertor, request);
        write("<table class=\"formTable\">");
        write(getEditFormContents(request, invertor));
        write("</table>");
    }
    write(QString("<input type=\"submit\" value=\"%1\">\n"
    "</form>\n").arg(tr("Edit")));
    if(action == "edit2") {
        /// second page doing the action and redirecting
        /// validation
        if(invertor.invertor > 0 && true) {
            emit saveInvertor(invertor);
            QString location = QString("/invertors?message=%1").arg(percentEncode(tr("Invertor edited.")));
            redirect(location);
        } else {
            write(QString("<p>%1</p>").arg(tr("Error: Invertor definition invalid (not edited).")));
        }
    }
}

/**
 * Delete invertor action
 */
void InvertorsController::actionDelete(HttpRequest& request) {
    write(QString("<h2>%1</h2>\n").arg(tr("Delete an invertor")));
        QString location;
        QByteArray id = request.getParameter("id");
        int idi = id.toInt();
        if(!id.isEmpty() && idi > 0) {
            emit deleteInvertor(idi);
            location = QString("/invertors?message=%1").arg(percentEncode(tr("Invertor deleted.")));
        } else {
            location = QString("/invertors?message=%1").arg(percentEncode(tr("Error: Invalid invertor ID. Invertor not deleted.")));
        }
        redirect(location);
}
