/**
 * @file
 * @copyright GNU Public License
 * @author Jiří Procházka
 * @author Petr Bravenec
*/

#include "logincontroller.h"
#include <QVariant>
#include <QDateTime>
#include <QCryptographicHash>
#include <QDebug>
#include "httpsession.h"
#include "static.h"

LoginController::LoginController() { 
    m_needs_access_data     = false;
    m_needs_change_settings = false;
}

void LoginController::servicePrivate(HttpRequest& request) {
    if(!request.getParameter("logout").isEmpty()) {
        actionLogout(request); return;
    } else if(request.getParameter("action").startsWith("chpw")) {
        actionChPW(request); return;
    }

    QByteArray retP = request.getParameter("returnPath");
    retP = (retP.isEmpty()) ? QByteArray("/") : QByteArray::fromPercentEncoding(retP);

    QString username = request.getParameter("username");
    QString password = request.getParameter("password");
    QString passsha1 = QCryptographicHash::hash(password.toAscii(), QCryptographicHash::Sha1).toHex();
    QList<DBT_USERS> users = Static::db->users(username);
    qDebug() << username << passsha1 << password;
    qDebug() << users.value(0).user 
             << users.value(0).password
             << "????";
    if (!username.isEmpty() && !users.isEmpty() && 
            users[0].user == username &&
            users[0].password == passsha1) {
        // Are we logged in?
        HttpSession session = getSession();
        session.set("username",username);
        if (users[0].can_access_data    ) { session.set("can_access_data",    true); } else { session.remove("can_access_data"); }
        if (users[0].can_change_settings) { session.set("can_change_settings",true); } else { session.remove("can_change_settings"); }
        redirect(QString(retP));
        return;
        }

    QString i ("<tr><td><label for=\"%2\">%1</label>: </td><td><input type=\"%3\" name=\"%2\" /></td></tr>\n");
    write(tr("<h2>You have to login to this page</h2>\n")
        + QString("<form method=\"post\"><table class=\"formTable\">\n")
        + i.arg(tr("Username"), "username", "text")
        + i.arg(tr("Password"), "password", "password")
        + QString("<tr><td></td><td><input type=\"hidden\" name=\"returnPath\" value=\"%1\" />"
        "<br/><input type=\"submit\" value=\"%2\"></td></tr>\n"
        "</table></form>\n").arg(QString(retP.toPercentEncoding()), tr(" OK "))
        );

    about();
}

/**
 * Log out action
 */
void LoginController::actionLogout(HttpRequest& request) {
    Q_UNUSED(request);
    HttpSession session = getSession();
    Static::sessionStore->removeSession(session);
    redirect("/");
}

/**
 * Change password action
 */
void LoginController::actionChPW(HttpRequest& request) {
    QByteArray action = request.getParameter("action");
    HttpSession session = getSession();
    if(!session.contains("username")) {
            QString location = QString("/login?message=%1").arg(percentEncode(tr("Need to log in to change password.")));
            redirect(location);
    }
    /// a form
    write(QString("<h3>%1</h3>\n").arg(tr("Change Password")));
    write("<form method=\"post\">\n"
        "<input type=\"hidden\" name=\"action\" value=\"chpw2\" />\n");
    QString pw = request.getParameter("pw");
    write(QString("<input type=\"password\" name=\"pw\" value=\"%1\" />").arg(pw));
    write(QString("<input type=\"submit\" value=\"%1\">\n"
    "</form>\n").arg(tr("Change")));
    if(action == "chpw2") {
        /// second page doing the action and redirecting
        /// validation
        if(!pw.isEmpty()) {
            DBT_USERS user;
            user.user = session.get("username").toString();
            user.password = pw;
            Static::db->saveUser(user);
            QString location = QString("/?message=%1").arg(percentEncode(tr("Password changed.")));
            redirect(location);
        } else {
            write(QString("<p>%1</p>").arg(tr("Error: Password invalid.")));
        }
    }
}
