/**
 * @file
 * @copyright GNU Public License
 * @author Petr Bravenec, Jiří Procházka
*/

#include "networkcontroller.h"
#include "static.h"
#include <QVariant>
#include <QDateTime>
#include <QFile>
#include <QRegExp>
#include <QDebug>
#include <QProcess>

#define CONFIG_NET      "/etc/conf.d/net"
#define CONFIG_HOSTNAME "/etc/conf.d/hostname"

NetworkController::NetworkController() {
}

void NetworkController::servicePrivate(HttpRequest& request) {
    QByteArray action = request.getParameter("action");
    if (action.startsWith("apply")) {
        actionApply(request); 
        return;
        }

    getConfigValues();

    write(QString(
        "<h2>%1</h2>\n"
        "<table class=\"formTable\">\n"
        "<form>\n"
        "<input type=\"hidden\" name=\"action\" value=\"apply\"/>\n"
        "<tr><td>%2</td><td><input type=\"text\"     name=\"hostname\" value=\"%3\"></td></tr>\n"
        "<tr><td>%4</td><td><input type=\"checkbox\" name=\"ipv4_dhcp\" value=\"Y\" %5></td></tr>\n"
        "<tr><td>%6</td><td><input type=\"text\" name=\"ipv4_addr\" value=\"%7\"></td></tr>\n"
        "<tr><td>%8</td><td><input type=\"text\" name=\"ipv4_mask\" value=\"%9\"></td></tr>\n"
        "<tr><td>%10</td><td><input type=\"text\" name=\"ipv4_router\" value=\"%11\"></td></tr>\n"
        "<tr><td>%12</td><td><input type=\"text\" name=\"ipv4_dns\" value=\"%13\"></td></tr>\n"
        "<tr><td></td><td><input type=\"submit\" value=\"OK\"></td></tr>\n"
        "</form></table>\n" )
        .arg(tr("Network settings"))
        .arg(tr("Hostname:"))           .arg(  m_hostname )
        .arg(tr("Use DHCP:"))           .arg( (m_ipv4_dhcp) ? "checked" : "" )
        .arg(tr("IPv4 address:"))       .arg(  m_ipv4_addr )
        .arg(tr("IPv4 mask:"))          .arg(  m_ipv4_mask )
        .arg(tr("IPv4 router:"))        .arg(  m_ipv4_router )
        .arg(tr("IPv4 dns server:"))    .arg(  m_ipv4_dns )
        .toUtf8()
        );

}


void NetworkController::getConfigValues() {
    m_hostname      = "hostname";
    m_ipv4_dhcp     = false;
    m_ipv4_addr     = "192.168.1.135";
    m_ipv4_mask     = "255.255.255.0";
    m_ipv4_router   = "192.168.1.1";
    m_ipv4_dns      = "8.8.8.8";

    QRegExp rx_dhcp     ("config_eth0=\"dhcp\"");
    QRegExp rx_addr     ("config_eth0=\"([0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3})\\snetmask\\s([0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3})\"");
    QRegExp rx_route    ("routes_eth0=\"default via ([0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3})\"");
    QRegExp rx_dns      ("dns_servers_eth0=\"([0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3})\"");
    QRegExp rx_hostname ("hostname=\"(.*)\"");

    QFile net(CONFIG_NET);
    if (!net.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qDebug() << "Could not open /etc/conf.d/net";
        return;
        }

    while (!net.atEnd()) {
        QString line = QString::fromUtf8(net.readLine()).trimmed();
        if (rx_dhcp.exactMatch(line)) {
            m_ipv4_dhcp = true;
            break;
            }
        if (rx_addr.exactMatch(line)) {
            m_ipv4_addr = rx_addr.cap(1);
            m_ipv4_mask = rx_addr.cap(2);
            continue;
            }
        if (rx_route.exactMatch(line)) {
            m_ipv4_router = rx_route.cap(1);
            continue;
            }
        if (rx_dns.exactMatch(line)) {
            m_ipv4_dns = rx_dns.cap(1);
            continue;
            }
        }

    net.close(); 

    QFile host(CONFIG_HOSTNAME);
    if (!host.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qDebug() << "Could not open /etc/conf.d/hostname";
        return;
        }

    while (!host.atEnd()) {
        QString line = QString::fromUtf8(host.readLine()).trimmed();
        if (rx_hostname.exactMatch(line)) {
            m_hostname = rx_hostname.cap(1);
            break;
            }
        }

    host.close(); 

}


void NetworkController::actionApply(HttpRequest& request) {
    m_hostname      = request.getParameter("hostname");
    m_ipv4_dhcp     = ( request.getParameter("ipv4_dhcp") == "Y") ? true : false;
    m_ipv4_addr     = request.getParameter("ipv4_addr");
    m_ipv4_mask     = request.getParameter("ipv4_mask");
    m_ipv4_router   = request.getParameter("ipv4_router");
    m_ipv4_dns      = request.getParameter("ipv4_dns");

    QFile host(CONFIG_HOSTNAME);
    if (!host.open(QIODevice::WriteOnly | QIODevice::Text)) {
        qDebug() << "Could not open /etc/conf.d/hostname for writing";
        write( tr("Sorry, it is not possible to write to /etc/conf.d/hostname").toUtf8() );
        return;
        }
    host.write("# This file is generated by fotobot application\n");
    host.write("# Your changes will be overwritten\n\n");
    host.write( QString("hostname=\"%1\"\n").arg(m_hostname).toUtf8() );
    host.close();

    QFile net(CONFIG_NET);
    if (!net.open(QIODevice::WriteOnly | QIODevice::Text)) {
        qDebug() << "Could not open /etc/conf.d/hostname for writing";
        write( tr("Sorry, it is not possible to write to /etc/conf.d/net").toUtf8() );
        return;
        }
    net.write("# This file is generated by fotobot application\n");
    net.write("# Your changes will be overwritten\n\n");
    if (m_ipv4_dhcp) {
        net.write( "dns_servers_eth0=\"8.8.8.8\"\n" );
        net.write( "config_eth0=\"dhcp\"\n" );
        net.write( "fallback_eth0=\"192.168.1.111 netmask 255.255.255.0\"\n" );
      } else {
        net.write( QString("config_eth0=\"%1 netmask %2\"\n").arg(m_ipv4_addr).arg(m_ipv4_mask).toUtf8() );
        net.write( QString("routes_eth0=\"%1\"\n").arg(m_ipv4_router).toUtf8() );
        net.write( QString("dns_servers_eth0=\"%1\"\n").arg(m_ipv4_dns).toUtf8() );
        }
    net.close();

    write( tr("<h2>Restarting....</h2>\n<p>Device is restarting now. Please wait few minutes and then refresh this page.</p>").toUtf8() );

    QProcess *restart = new QProcess();
    restart->start("/usr/bin/fotobot-reboot");

}

