/**
 * @file
 * @copyright GNU Public License
 * @author Petr Bravenec, Jiří Procházka
*/

#include "userscontroller.h"
#include "invertorscontroller.h"
#include "static.h"
#include <QVariant>
#include <QDateTime>

UsersController::UsersController() {
    qRegisterMetaType<DBT_USERS>();
    connect(this,       SIGNAL(saveUser(DBT_USERS)), 
            Static::db,   SLOT(saveUser(DBT_USERS)), 
            Qt::QueuedConnection);
    connect(this,       SIGNAL(deleteUser(QString)), 
            Static::db,   SLOT(deleteUser(QString)), 
            Qt::QueuedConnection);
}

void UsersController::servicePrivate(HttpRequest& request) {
    QByteArray action = request.getParameter("action");
    if (action.startsWith("add")) {
        actionAdd(request); return;
      } else if(action.startsWith("edit")) {
        actionEdit(request); return;
      } else if(action.startsWith("delete")) {
        actionDelete(request); return;
        }

    /** Default action/page */

    QList<DBT_USERS> users = Static::db->users();
    write(QString("<h2>%1</h2>\n").arg(tr("Users")));
    write(QString("<a href=\"/users?action=add\">%1</a>\n").arg(tr("Add an user")));
    write("<table>\n");
    write(QString("<tr>"));
    write(QString("<th></th>"));
    write(QString("<th>%1</th>").arg("Username"));
    write(QString("<th>%1</th>").arg("Can acess data"));
    write(QString("<th>%1</th>").arg("Can change settings"));
    write(QString("</tr>"));

    QList<DBT_USERS>::const_iterator i;
    for(i = users.constBegin(); i != users.constEnd(); ++i) {
        const DBT_USERS& l = *i;
        write(QString("<tr>"));
        write(QString("<td>"
                "<a href=\"/users?action=edit&user=%1\">%2</a> "
                "<a href=\"/users?action=delete&user=%1\">%3</a>"
                "</td>")
                .arg(l.user)
                .arg(tr("Edit"))
                .arg(tr("Delete"))
                );
        write(QString("<td>%1</td>").arg( (l.user) ));
        write(QString("<td>%1</td>").arg( (l.can_access_data)     ? "&#x2713;" : ""));
        write(QString("<td>%2</td>").arg( (l.can_change_settings) ? "&#x2713;" : ""));
        write(QString("</td>\n"));
        }

    write("</table>\n");
}

QString UsersController::getEditFormContents(const DBT_USERS& user) {
    QString s;
    s += QString("<input type=\"hidden\" name=\"id\" value=\"%1\" />\n").arg(user.user);
    s += QString("<tr>");
    s += QString("  <td><label for=\"user\">%1</td>").arg(tr("Username"));
    s += QString("  <td><input type=\"text\" name=\"user\" id=\"user\" value=\"%1\"></td>").arg(user.user);
    s += QString("  </tr>");

    s += QString("<tr>");
    s += QString("  <td><label for=\"password\">%1</td>").arg(tr("Password"));
    s += QString("  <td><input type=\"text\" name=\"password\" id=\"password\"></td>");
    s += QString("  </tr>");

    s += QString("<tr>");
    s += QString("  <td><label for=\"can_access_data\">%1</td>").arg(tr("Can access data"));
    s += QString("  <td><input type=\"checkbox\" name=\"can_access_data\" id=\"can_access_data\" %1></td>").arg((user.can_access_data) ? "checked=\"checked\"" : "");
    s += QString("  </tr>");

    s += QString("<tr>");
    s += QString("  <td><label for=\"can_change_settings\">%1</td>").arg(tr("Can change settings"));
    s += QString("  <td><input type=\"checkbox\" name=\"can_change_settings\" id=\"can_change_settings\" %1></td>").arg((user.can_change_settings) ? "checked=\"checked\"" : "");
    s += QString("  </tr>");

    return s;
}

void UsersController::readForm(DBT_USERS& user, HttpRequest& request) {
    user.id                     =  request.getParameter("id");
    user.user                   =  request.getParameter("user");
    user.password               =  request.getParameter("password");
    user.can_access_data        = !request.getParameter("can_access_data").isEmpty();
    user.can_change_settings    = !request.getParameter("can_change_settings").isEmpty();
}


/**
 * Add an user action
 */
void UsersController::actionAdd(HttpRequest& request) {
    QByteArray action = request.getParameter("action");
    write(QString("<h3>%1</h3>\n").arg(tr("Add an user")));
    write("<form method=\"post\">\n"
        "<input type=\"hidden\" name=\"action\" value=\"add2\" />\n");
    DBT_USERS user;
    readForm(user, request);
    write("<table class=\"formTable\">");
    write(getEditFormContents(user));
    write("</table>");
    write(QString("<input type=\"submit\" value=\"%1\">\n"
    "</form>\n").arg(tr("Add")));
    if (action == "add2") {
        /// second page doing the action and redirecting
        user.id = QString();
        emit saveUser(user);
        QString location = QString("/users?message=%1").arg(percentEncode(tr("User added.")));
        redirect(location);
        }
}


/**
 * Edit user action
 */
void UsersController::actionEdit(HttpRequest& request) {
    QByteArray action = request.getParameter("action");
    DBT_USERS user;
    /// a form
    write(QString("<h3>%1</h3>\n").arg(tr("Edit an user")));
    write("<form method=\"post\">\n"
        "<input type=\"hidden\" name=\"action\" value=\"edit2\" />\n");
    if (action == "edit") {
        QByteArray userid = request.getParameter("user");
        if(userid.isEmpty()) {
            redirect(QString("/user?message=%1").arg(percentEncode(tr("Error: Invalid user ID."))));
            return;
            }
        QList<DBT_USERS> users = Static::db->users(userid);
        if(users.empty()) {
            redirect(QString("/users?message=%1").arg(percentEncode(tr("Error: no such user found."))));
            return;
            }
        write("<table class=\"formTable\">");
        write(getEditFormContents(users.at(0)));
        write("</table>");
      } else {
        readForm(user, request);
        write("<table class=\"formTable\">");
        write(getEditFormContents(user));
        write("</table>");
        }
    write(QString("<input type=\"submit\" value=\"%1\"></form>\n").arg(tr("Save")));
    if (action == "edit2") {
        /// second page doing the action and redirecting
        if(!user.id.isEmpty()) {
            emit saveUser(user);
            QString location = QString("/users?message=%1").arg(percentEncode(tr("User edited.")));
            redirect(location);
          } else {
            write(QString("<p>%1</p>").arg(tr("Error: User definition invalid.")));
            }
        }
}

/**
 * Delete user action
 */
void UsersController::actionDelete(HttpRequest& request) {
    write(QString("<h3>%1</h3>\n").arg(tr("Delete an user ")));
    QString location;
    QByteArray userid = request.getParameter("user");
    if(!userid.isEmpty()) {
        emit deleteUser(userid);
        location = QString("/users?message=%1").arg(percentEncode(tr("User deleted.")));
      } else {
        location = QString("/users?message=%1").arg(percentEncode(tr("Error: Invalid user ID. User not deleted.")));
        }
    redirect(location);
}

