/**
 * @file
 * @author Petr Bravenec, Jiří Procházka
 * @copyright GNU Public License
 */

#include "requestmapper.h"
#include "static.h"
#include "staticfilecontroller.h"

#include "controller/mainpagecontroller.h"
#include "controller/logincontroller.h"
#include "controller/settingscontroller.h"
#include "controller/exportcontroller.h"
#include "controller/statuscontroller.h"
#include "controller/linescontroller.h"
#include "controller/invertorscontroller.h"
#include "controller/interfaceboxescontroller.h"
#include "controller/geocontroller.h"
#include "controller/userscontroller.h"
#ifdef GENTOO
#include "controller/networkcontroller.h"
#endif

RequestMapper::RequestMapper(QObject* parent)
    :HttpRequestHandler(parent) {}

void RequestMapper::service(HttpRequest& request, HttpResponse& response) {
    QByteArray path=request.getPath();

    qWarning() << "Web request:" << path;

    if (path.startsWith("/login")) {
        LoginController().service(request, response);
    }

    else if (path == "/") {
        MainPageController().service(request, response);
    }

    else if (path.startsWith("/settings")) {
        SettingsController().service(request, response);
    }

    else if (path.startsWith("/export")) {
        ExportController().service(request, response);
    }

    else if (path.startsWith("/lines")) {
        LinesController().service(request, response);
    }

    else if (path.startsWith("/invertors")) {
        InvertorsController().service(request, response);
    }

    else if (path.startsWith("/interfaceboxes")) {
        InterfaceboxesController().service(request, response);
    }

    else if (path.startsWith("/geocoordinates")) {
        GeoController().service(request, response);
    }

#ifdef GENTOO
    else if (path.startsWith("/network")) {
        NetworkController().service(request, response);
    }
#endif

    else if (path.startsWith("/status")) {
        StatusController().service(request, response);
    }

    else if (path.startsWith("/users")) {
        UsersController().service(request, response);
    }

    // All other pathes are mapped to the static file controller.
    else {
        Static::staticFileController->service(request, response);
    }
}
