/**
 * @file
 * @author Petr Bravenec, Jiří Procházka
 * @copyright GNU Public License
 */

#ifndef _WEBPAGE_H_
#define _WEBPAGE_H_

#include "httprequest.h"
#include "httpresponse.h"
#include "httpsession.h"
#include "static.h"
#include "mrequesthandler.h"

/**
  This controller demonstrates how to use sessions.
*/

class WebPage : public MRequestHandler {
    Q_OBJECT
    Q_DISABLE_COPY(WebPage);
  public:
    /** Constructor */
    WebPage();

    virtual void servicePrivate(HttpRequest& request) = 0;
    virtual void service       (HttpRequest& request, HttpResponse& response);
    void    redirect(const QString& location);
    static QString percentEncode(const QString& s);

  protected:
    QString    m_javascript;

    void    setStatus(int code, QByteArray description)  { m_response->setStatus(code, description); }
    void    setHeader(QByteArray  name, QByteArray value) { m_response->setHeader(name, value); }
    void    setHeader(QByteArray  name, int        value) { m_response->setHeader(name, value); }
    void    write    (QByteArray  data) { m_body.append(data); }
    void    write    (QString     data) { m_body.append(data.toUtf8()); }
    void    write    (const char* data) { m_body.append(data); }
    void    about    ();
    HttpSession getSession() { return Static::sessionStore->getSession(*m_request,*m_response); }

    virtual void  writeHeader(HttpRequest& request, HttpResponse& response);
    virtual void  writeFooter(HttpRequest& request, HttpResponse& response);

  private:
    HttpResponse *m_response;
    HttpRequest  *m_request;
    QByteArray    m_body;
};

#endif // SESSIONCONTROLLER_H
